window.Gleam.initSearch([{"doc":"pevensie","title":"pevensie","content":"# Pevensie\n\n[![Package Version](https://img.shields.io/hexpm/v/pevensie)](https://hex.pm/packages/pevensie)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/pevensie/)\n\nThe backend application framework for Gleam!\n\n## Getting Started\n\n```sh\ngleam add pevensie\n```\n\nPevensie uses a driver-based architecture, so most modules in the framework will\nrequire you to provide a driver for them to use. The drivers provided with\nPevensie are:\n\n- [Postgres](/pevensie/drivers/postgres.html) - A driver for PostgreSQL\n- [Redis](/pevensie/drivers/redis.html) - A driver for Redis (coming soon)\n- [ETS](/pevensie/drivers/ets.html) - A driver for ETS, Erlang&#39;s in-memory key-value store (coming soon)\n\nThe hope is that other first- and third-party drivers will be available in the\nfuture.\n\n&gt; Note: database-based drivers will require migrations to be run before using\n&gt; them. See the documentation for your chosen driver for more information.\n\n## Example\n\nThe below example uses Pevensie Auth to create a user, create a session for that\nuser, and create a signed cookie that can be used to authenticate future requests.\n\n```gleam\nimport gleam/dynamic.{type DecodeError}\nimport gleam/json\nimport gleam/option.{None, Some}\nimport pevensie/auth\nimport pevensie/drivers/postgres\n\ntype UserMetadata {\n  UserMetadata(name: String, age: Int)\n}\n\nfn user_metadata_encoder(user_metadata: UserMetadata) -&gt; json.Json {\n  json.object([\n    #(&quot;name&quot;, json.string(user_metadata.name)),\n    #(&quot;age&quot;, json.int(user_metadata.age)),\n  ])\n}\n\nfn user_metadata_decoder() -&gt; Result(UserMetadata, List(DecodeError)) {\n  decode.into({\n    use name &lt;- decode.parameter\n    use age &lt;- decode.parameter\n    Ok(UserMetadata(name: name, age: age))\n  })\n}\n\npub fn main() {\n  let config = postgres.default_config()\n  let driver = postgres.new_auth_driver(config)\n  let pevensie_auth = auth.new(\n    driver,\n    user_metadata_decoder,\n    user_metadata_encoder,\n    &quot;super secret cookie signing key&quot;,\n  )\n\n  // Set up the Postgres connection pool\n  let assert Ok(pevensie_auth) = auth.connect(pevensie_auth)\n\n  // Create a user\n  let assert Ok(user) =\n    pevensie_auth\n    |&gt; auth.create_user_with_email(\n      &quot;lucy@pevensie.dev&quot;,\n      &quot;password&quot;,\n      UserMetadata(name: &quot;Lucy Pevensie&quot;, age: 8),\n    )\n\n  // Create a session for the user\n  let assert Ok(session) =\n    pevensie_auth\n    |&gt; auth.create_session(\n      user.id,\n      None,\n      None,\n      Some(24 * 60 * 60),\n    )\n\n  // Create a signed cookie for the session\n  let assert Ok(session_cookie) =\n    pevensie_auth\n    |&gt; auth.create_session_cookie(session)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/pevensie&gt;.\n\n## Why v0?\n\nThe Gleam community generally holds the (correct) opinion that libraries should not be\nreleased until they&#39;re ready. Thus, version numbers pre-v1 generally don&#39;t exist.\n\nHowever, Pevensie is starting with v0 versions for a couple of reasons, which can be\nseen below.\n\n&gt; [!CAUTION]\n&gt; This is a beta version of Pevensie, and is not yet ready for production use.\n&gt; Consider nothing to be stable, including database migrations. They, or the\n&gt; format they use, may change at any time before v1.\n\nWhile the project is in v0, patch versions will be used for bug fixes, and minor\nversions will be used for new features, which may be backwards-incompatible.\n\n### Security\n\nI&#39;ve never written an authentication library before, and I&#39;m not a security expert.\nThere&#39;s a non-zero chance that I&#39;ve made a mistake somewhere, and there&#39;s a security\nvulnerability somewhere in Pevensie Auth.\n\nBy releasing a v0 version of Pevensie, I give the community a chance to find and flag\nany security issues before they&#39;re released into the wild under the guise of a\n&#39;production-ready&#39; framework.\n\n### The API\n\nWhile it&#39;s currently fairly small, Pevensie&#39;s architecture is somewhat complex, aiming\nto be opinionated where it&#39;s needed, but still flexible enough to allow users to choose\nthe majority of their tech stack (hence the driver-based design).\n\nI can&#39;t guarantee that this approach is the best, and the best way to validate that is\nto see how the community use the framework. Having a v0 means that both the\npublic-facing API and the driver API can change quickly until some sort of stable\nconfiguration is reached.\n\n## Feedback and feature requests\n\nIf you have any feedback or feature requests, please open an issue on the\n[Pevensie GitHub repository](https://github.com/Pevensie/pevensie).\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n\n### V1 checklist\n\n&gt; [!NOTE]\n&gt; This section will be replaced with a proper GitHub project board shortly.\n\n- [ ] Add more decoders for JSON representation of provided types\n- [ ] Decide on a stable architecture\n- [ ] Add more drivers (Redis, ETS)\n- [ ] Write unit and integration tests\n- [ ] Add Pevensie Jobs for async/background tasks\n- [ ] Email confirmation tokens\n- [ ] Email change tokens\n","url":"index.html"},{"doc":"pevensie/drivers","title":"ConnectError","content":"pub type ConnectError(driver_error) {\n  ConnectDriverError(driver_error)\n  AlreadyConnected\n}\n\nConnectDriverError(driver_error)\n\n\nAlreadyConnected\n\n\nSynonyms:\npevensie/drivers.ConnectError\npevensie/drivers ConnectError","url":"pevensie/drivers.html#ConnectError"},{"doc":"pevensie/drivers","title":"ConnectFunction","content":"pub type ConnectFunction(driver, driver_error) =\n  fn(driver) -&gt; Result(driver, ConnectError(driver_error))\n A function that connects the driver. This may\n set up any connections or perform any other setup\n required to make the driver ready to use.\n\nSynonyms:\npevensie/drivers.ConnectFunction\npevensie/drivers ConnectFunction","url":"pevensie/drivers.html#ConnectFunction"},{"doc":"pevensie/drivers","title":"Connected","content":"pub type Connected\n\n\nSynonyms:\npevensie/drivers.Connected\npevensie/drivers Connected","url":"pevensie/drivers.html#Connected"},{"doc":"pevensie/drivers","title":"Disabled","content":"pub type Disabled\n\n\nSynonyms:\npevensie/drivers.Disabled\npevensie/drivers Disabled","url":"pevensie/drivers.html#Disabled"},{"doc":"pevensie/drivers","title":"DisconnectError","content":"pub type DisconnectError(driver_error) {\n  DisconnectDriverError(driver_error)\n  NotConnected\n}\n\nDisconnectDriverError(driver_error)\n\n\nNotConnected\n\n\nSynonyms:\npevensie/drivers.DisconnectError\npevensie/drivers DisconnectError","url":"pevensie/drivers.html#DisconnectError"},{"doc":"pevensie/drivers","title":"DisconnectFunction","content":"pub type DisconnectFunction(driver, driver_error) =\n  fn(driver) -&gt; Result(driver, DisconnectError(driver_error))\n A function that disconnects the driver. This may\n tear down any connections or perform any other cleanup\n required once the driver is no longer needed.\n\nSynonyms:\npevensie/drivers.DisconnectFunction\npevensie/drivers DisconnectFunction","url":"pevensie/drivers.html#DisconnectFunction"},{"doc":"pevensie/drivers","title":"Disconnected","content":"pub type Disconnected\n\n\nSynonyms:\npevensie/drivers.Disconnected\npevensie/drivers Disconnected","url":"pevensie/drivers.html#Disconnected"},{"doc":"pevensie/drivers","title":"pevensie/drivers","content":"","url":"pevensie/drivers.html"},{"doc":"pevensie/net","title":"IpAddress","content":"pub type IpAddress {\n  IpV4(Int, Int, Int, Int)\n  IpV6(Int, Int, Int, Int, Int, Int, Int, Int)\n}\n An IP address. Can be either an IPv4 or IPv6 address. Data is stored\n as a record of integers. To display an IP address, use the\n [`format_ip_address`](#format_ip_address) function.\nIpV4(Int, Int, Int, Int)\n\n\nIpV6(Int, Int, Int, Int, Int, Int, Int, Int)\n\n\nSynonyms:\npevensie/net.IpAddress\npevensie/net IpAddress","url":"pevensie/net.html#IpAddress"},{"doc":"pevensie/net","title":"format_ip_address","content":"pub fn format_ip_address(ip: IpAddress) -&gt; String\n Formats an IP address as a string. This is useful for displaying\n IP addresses in a user-friendly way, or for inserting them into\n a database.\nSynonyms:\npevensie/net.format_ip_address\npevensie/net format_ip_address","url":"pevensie/net.html#format_ip_address"},{"doc":"pevensie/net","title":"parse_ip_address","content":"pub fn parse_ip_address(ip: String) -&gt; Result(IpAddress, Nil)\n Parses an IP address from a string. Returns an error if the string\n is not a valid IP address. Accepts relaxed IPv4 and IPv6 formats,\n such as `127.1` and `2001:db8::1`.\nSynonyms:\npevensie/net.parse_ip_address\npevensie/net parse_ip_address","url":"pevensie/net.html#parse_ip_address"},{"doc":"pevensie/net","title":"pevensie/net","content":" `pevensie/net` contains convencience functions for working with\n IP addresses and other networking-related tasks.","url":"pevensie/net.html"},{"doc":"pevensie/auth","title":"AppMetadata","content":"pub opaque type AppMetadata\n Internal metadata used by Pevensie. Contains user information such as\n OAuth tokens, etc.\n\nSynonyms:\npevensie/auth.AppMetadata\npevensie/auth AppMetadata","url":"pevensie/auth.html#AppMetadata"},{"doc":"pevensie/auth","title":"AuthDriver","content":"pub type AuthDriver(driver, driver_error, user_metadata) {\n  AuthDriver(\n    driver: driver,\n    connect: ConnectFunction(driver, driver_error),\n    disconnect: DisconnectFunction(driver, driver_error),\n    list_users: ListUsersFunction(\n      driver,\n      driver_error,\n      user_metadata,\n    ),\n    create_user: CreateUserFunction(\n      driver,\n      driver_error,\n      user_metadata,\n    ),\n    update_user: UpdateUserFunction(\n      driver,\n      driver_error,\n      user_metadata,\n    ),\n    delete_user: DeleteUserFunction(\n      driver,\n      driver_error,\n      user_metadata,\n    ),\n    get_session: GetSessionFunction(driver, driver_error),\n    create_session: CreateSessionFunction(driver, driver_error),\n    delete_session: DeleteSessionFunction(driver, driver_error),\n    create_one_time_token: CreateOneTimeTokenFunction(\n      driver,\n      driver_error,\n    ),\n    validate_one_time_token: ValidateOneTimeTokenFunction(\n      driver,\n      driver_error,\n    ),\n    use_one_time_token: UseOneTimeTokenFunction(\n      driver,\n      driver_error,\n    ),\n    delete_one_time_token: DeleteOneTimeTokenFunction(\n      driver,\n      driver_error,\n    ),\n  )\n}\n\nAuthDriver(\n  driver: driver,\n  connect: ConnectFunction(driver, driver_error),\n  disconnect: DisconnectFunction(driver, driver_error),\n  list_users: ListUsersFunction(\n    driver,\n    driver_error,\n    user_metadata,\n  ),\n  create_user: CreateUserFunction(\n    driver,\n    driver_error,\n    user_metadata,\n  ),\n  update_user: UpdateUserFunction(\n    driver,\n    driver_error,\n    user_metadata,\n  ),\n  delete_user: DeleteUserFunction(\n    driver,\n    driver_error,\n    user_metadata,\n  ),\n  get_session: GetSessionFunction(driver, driver_error),\n  create_session: CreateSessionFunction(driver, driver_error),\n  delete_session: DeleteSessionFunction(driver, driver_error),\n  create_one_time_token: CreateOneTimeTokenFunction(\n    driver,\n    driver_error,\n  ),\n  validate_one_time_token: ValidateOneTimeTokenFunction(\n    driver,\n    driver_error,\n  ),\n  use_one_time_token: UseOneTimeTokenFunction(\n    driver,\n    driver_error,\n  ),\n  delete_one_time_token: DeleteOneTimeTokenFunction(\n    driver,\n    driver_error,\n  ),\n)\n\n\nSynonyms:\npevensie/auth.AuthDriver\npevensie/auth AuthDriver","url":"pevensie/auth.html#AuthDriver"},{"doc":"pevensie/auth","title":"CreateError","content":"pub type CreateError(auth_driver_error) {\n  CreateDriverError(auth_driver_error)\n  CreatedTooFewRecords\n  CreatedTooManyRecords\n  CreateHashError(argus.HashError)\n  CreateInternalError(String)\n}\n\nCreateDriverError(auth_driver_error)\n\n\nCreatedTooFewRecords\n\n\nCreatedTooManyRecords\n\n\nCreateHashError(argus.HashError)\n\n\nCreateInternalError(String)\n\n\nSynonyms:\npevensie/auth.CreateError\npevensie/auth CreateError","url":"pevensie/auth.html#CreateError"},{"doc":"pevensie/auth","title":"DeleteError","content":"pub type DeleteError(auth_driver_error) {\n  DeleteDriverError(auth_driver_error)\n  DeletedTooFewRecords\n  DeletedTooManyRecords\n  DeleteInternalError(String)\n}\n\nDeleteDriverError(auth_driver_error)\n\n\nDeletedTooFewRecords\n\n\nDeletedTooManyRecords\n\n\nDeleteInternalError(String)\n\n\nSynonyms:\npevensie/auth.DeleteError\npevensie/auth DeleteError","url":"pevensie/auth.html#DeleteError"},{"doc":"pevensie/auth","title":"GetError","content":"pub type GetError(auth_driver_error) {\n  GetDriverError(auth_driver_error)\n  GotTooFewRecords\n  GotTooManyRecords\n  GetInternalError(String)\n}\n\nGetDriverError(auth_driver_error)\n\n\nGotTooFewRecords\n\n\nGotTooManyRecords\n\n\nGetInternalError(String)\n\n\nSynonyms:\npevensie/auth.GetError\npevensie/auth GetError","url":"pevensie/auth.html#GetError"},{"doc":"pevensie/auth","title":"LogInError","content":"pub type LogInError(auth_driver_error) {\n  LogInUserError(GetError(auth_driver_error))\n  LogInSessionError(CreateError(auth_driver_error))\n}\n\nLogInUserError(GetError(auth_driver_error))\n\n\nLogInSessionError(CreateError(auth_driver_error))\n\n\nSynonyms:\npevensie/auth.LogInError\npevensie/auth LogInError","url":"pevensie/auth.html#LogInError"},{"doc":"pevensie/auth","title":"OneTimeTokenType","content":"pub type OneTimeTokenType {\n  PasswordReset\n}\n\nPasswordReset\n\n\nSynonyms:\npevensie/auth.OneTimeTokenType\npevensie/auth OneTimeTokenType","url":"pevensie/auth.html#OneTimeTokenType"},{"doc":"pevensie/auth","title":"PevensieAuth","content":"pub opaque type PevensieAuth(\n  driver,\n  driver_error,\n  user_metadata,\n  connected,\n)\n The entrypoint to the Pevensie Auth API. This type is used when using\n the majority of the functions in `pevensie/auth`.\n\n You must connect your Pevensie Auth instance before using it. This\n allows your driver to perform any setup required, such as creating\n a database connection pool.\n\n Create a new `PevensieAuth` instance using the\n [`new`](#new) function.\n\n \n import pevensie/auth.{type PevensieAuth}\n\n pub fn main() {\n   let pevensie_auth = auth.new(\n     postgres.new_auth_driver(postgres.default_config()),\n     user_metadata_decoder,\n     user_metadata_encoder,\n     &quot;super secret signing key&quot;,\n   )\n   // ...\n }\n \n\nSynonyms:\npevensie/auth.PevensieAuth\npevensie/auth PevensieAuth","url":"pevensie/auth.html#PevensieAuth"},{"doc":"pevensie/auth","title":"Session","content":"pub type Session {\n  Session(\n    id: String,\n    created_at: Time,\n    expires_at: Option(Time),\n    user_id: String,\n    ip: Option(IpAddress),\n    user_agent: Option(String),\n  )\n}\n A Pevensie session. Sessions are used to identify users and\n provide a way to track their activity.\n\n Fields:\n - `id`: The session&#39;s ID (unique).\n - `created_at`: The time the session was created.\n - `expires_at`: The time the session will expire.\n - `user_id`: The ID of the user associated with the session.\n - `ip`: The IP address of the user associated with the session.\n - `user_agent`: The user agent of the user associated with the session.\n\n When searching for sessions, auth drivers will check for IP and user\n agent matches if the `ip` and `user_agent` fields are not `None`.\nSession(\n  id: String,\n  created_at: Time,\n  expires_at: Option(Time),\n  user_id: String,\n  ip: Option(IpAddress),\n  user_agent: Option(String),\n)\n\n\nSynonyms:\npevensie/auth.Session\npevensie/auth Session","url":"pevensie/auth.html#Session"},{"doc":"pevensie/auth","title":"UpdateError","content":"pub type UpdateError(auth_driver_error) {\n  UpdateDriverError(auth_driver_error)\n  UpdatedTooFewRecords\n  UpdatedTooManyRecords\n  UpdateHashError(argus.HashError)\n  UpdateInternalError(String)\n}\n\nUpdateDriverError(auth_driver_error)\n\n\nUpdatedTooFewRecords\n\n\nUpdatedTooManyRecords\n\n\nUpdateHashError(argus.HashError)\n\n\nUpdateInternalError(String)\n\n\nSynonyms:\npevensie/auth.UpdateError\npevensie/auth UpdateError","url":"pevensie/auth.html#UpdateError"},{"doc":"pevensie/auth","title":"UpdateField","content":"pub type UpdateField(a) {\n  Set(a)\n  Ignore\n}\n A field to be updated in the database. Use the `Set` constructor to set a\n field to a value, and the `Ignore` constructor to leave the field unchanged.\nSet(a)\n\n\nIgnore\n\n\nSynonyms:\npevensie/auth.UpdateField\npevensie/auth UpdateField","url":"pevensie/auth.html#UpdateField"},{"doc":"pevensie/auth","title":"User","content":"pub type User(user_metadata) {\n  User(\n    id: String,\n    created_at: Time,\n    updated_at: Time,\n    deleted_at: Option(Time),\n    role: Option(String),\n    email: String,\n    password_hash: Option(String),\n    email_confirmed_at: Option(Time),\n    phone_number: Option(String),\n    phone_number_confirmed_at: Option(Time),\n    last_sign_in: Option(Time),\n    app_metadata: AppMetadata,\n    user_metadata: user_metadata,\n    banned_until: Option(Time),\n  )\n}\n Pevensie&#39;s user type. Users can be identified by ID, email, or phone number - \n all of which are unique.\n\n Fields:\n - `id`: The user&#39;s ID (unique).\n - `created_at`: The time the user was created.\n - `updated_at`: The time the user was last updated.\n - `deleted_at`: The time the user was deleted.\n - `role`: The user&#39;s role. This can be set to any string, and is not used by\n   Pevensie.\n - `email`: The user&#39;s email address.\n - `password_hash`: The user&#39;s hashed password.\n - `app_metadata`: Pevensie&#39;s internal metadata about the user.\n - `user_metadata`: Custom user metadata, such as username, avatar, etc. This\n   is not used by Pevensie.\n - `last_sign_in`: The time the user last signed in.\n\n The `user_metadata` field is a generic field that can be used to store any\n custom data about the user. When creating a [`PevensieAuth`](/pevensie/auth.html#PevensieAuth)\n instance, you will need to provide a `user_metadata` decoder and encoder.\n\n A few fields currently unused, but will be used in the future:\n - `email_confirmed_at`: The time the user confirmed their email address.\n - `phone_number_confirmed_at`: The time the user confirmed their phone number.\n - `banned_until`: The time the user will be banned.\nUser(\n  id: String,\n  created_at: Time,\n  updated_at: Time,\n  deleted_at: Option(Time),\n  role: Option(String),\n  email: String,\n  password_hash: Option(String),\n  email_confirmed_at: Option(Time),\n  phone_number: Option(String),\n  phone_number_confirmed_at: Option(Time),\n  last_sign_in: Option(Time),\n  app_metadata: AppMetadata,\n  user_metadata: user_metadata,\n  banned_until: Option(Time),\n)\n\n\nSynonyms:\npevensie/auth.User\npevensie/auth User","url":"pevensie/auth.html#User"},{"doc":"pevensie/auth","title":"UserCreate","content":"pub type UserCreate(user_metadata) {\n  UserCreate(\n    role: Option(String),\n    email: String,\n    password_hash: Option(String),\n    email_confirmed_at: Option(Time),\n    phone_number: Option(String),\n    phone_number_confirmed_at: Option(Time),\n    last_sign_in: Option(Time),\n    app_metadata: AppMetadata,\n    user_metadata: user_metadata,\n  )\n}\n A user to be inserted into the database. See [`User`](#User) for fields.\nUserCreate(\n  role: Option(String),\n  email: String,\n  password_hash: Option(String),\n  email_confirmed_at: Option(Time),\n  phone_number: Option(String),\n  phone_number_confirmed_at: Option(Time),\n  last_sign_in: Option(Time),\n  app_metadata: AppMetadata,\n  user_metadata: user_metadata,\n)\n\n\nSynonyms:\npevensie/auth.UserCreate\npevensie/auth UserCreate","url":"pevensie/auth.html#UserCreate"},{"doc":"pevensie/auth","title":"UserSearchFields","content":"pub type UserSearchFields {\n  UserSearchFields(\n    id: Option(List(String)),\n    email: Option(List(String)),\n    phone_number: Option(List(String)),\n  )\n}\n A set of fields to use when searching for or listing users.\n Each field can be set to a list of values to search for, to\n search for multiple values at once.\n\n Different drivers may handle search fields differently. See the\n documentation for the specific driver for more information.\n\n Can be constructed more easily using\n [`default_user_search_fields`](#default_user_search_fields).\nUserSearchFields(\n  id: Option(List(String)),\n  email: Option(List(String)),\n  phone_number: Option(List(String)),\n)\n\n\nSynonyms:\npevensie/auth.UserSearchFields\npevensie/auth UserSearchFields","url":"pevensie/auth.html#UserSearchFields"},{"doc":"pevensie/auth","title":"UserUpdate","content":"pub type UserUpdate(user_metadata) {\n  UserUpdate(\n    role: UpdateField(Option(String)),\n    email: UpdateField(String),\n    password_hash: UpdateField(Option(String)),\n    email_confirmed_at: UpdateField(Option(Time)),\n    phone_number: UpdateField(Option(String)),\n    phone_number_confirmed_at: UpdateField(Option(Time)),\n    last_sign_in: UpdateField(Option(Time)),\n    app_metadata: UpdateField(AppMetadata),\n    user_metadata: UpdateField(user_metadata),\n  )\n}\n A user to be updated in the database. See [`User`](#User) for fields.\n\n Can be constructed more easily using\n [`default_user_update`](#default_user_update).\nUserUpdate(\n  role: UpdateField(Option(String)),\n  email: UpdateField(String),\n  password_hash: UpdateField(Option(String)),\n  email_confirmed_at: UpdateField(Option(Time)),\n  phone_number: UpdateField(Option(String)),\n  phone_number_confirmed_at: UpdateField(Option(Time)),\n  last_sign_in: UpdateField(Option(Time)),\n  app_metadata: UpdateField(AppMetadata),\n  user_metadata: UpdateField(user_metadata),\n)\n\n\nSynonyms:\npevensie/auth.UserUpdate\npevensie/auth UserUpdate","url":"pevensie/auth.html#UserUpdate"},{"doc":"pevensie/auth","title":"app_metadata_encoder","content":"pub fn app_metadata_encoder(app_metadata: AppMetadata) -&gt; Json\n Encodes an [`AppMetadata`](#AppMetadata) value to JSON.\nSynonyms:\npevensie/auth.app_metadata_encoder\npevensie/auth app_metadata_encoder","url":"pevensie/auth.html#app_metadata_encoder"},{"doc":"pevensie/auth","title":"connect","content":"pub fn connect(\n  pevensie_auth: PevensieAuth(a, b, c, Disconnected),\n) -&gt; Result(PevensieAuth(a, b, c, Connected), ConnectError(b))\n Runs setup for your chosen auth driver and returns a connected\n [`PevensieAuth`](#PevensieAuth) instance.\n\n This function must be called before using any other functions in\n the Pevensie Auth API. Attempting to use the API before calling\n `connect` will result in a compile error.\n\n \n import pevensie/auth.{type PevensieAuth}\n\n pub fn main() {\n   // ...\n   let assert Ok(pevensie_auth) = auth.connect(pevensie_auth)\n   // ...\n }\n \nSynonyms:\npevensie/auth.connect\npevensie/auth connect","url":"pevensie/auth.html#connect"},{"doc":"pevensie/auth","title":"create_one_time_token","content":"pub fn create_one_time_token(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n  token_type token_type: OneTimeTokenType,\n  ttl_seconds ttl_seconds: Int,\n) -&gt; Result(String, CreateError(b))\n\nSynonyms:\npevensie/auth.create_one_time_token\npevensie/auth create_one_time_token","url":"pevensie/auth.html#create_one_time_token"},{"doc":"pevensie/auth","title":"create_session","content":"pub fn create_session(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n  ip ip: Option(IpAddress),\n  user_agent user_agent: Option(String),\n  ttl_seconds ttl_seconds: Option(Int),\n) -&gt; Result(Session, CreateError(b))\n Create a new session for a user. If IP address or user agent are provided,\n they will be stored alongside the session, and must be provided when\n fetching the session later.\n\n You can optionally set a TTL for the session, which will cause the session\n to expire. Set `ttl_seconds` to `None` to never expire the session.\n\n You can optionally delete any other active sessions for the user. This may\n be useful if you want to ensure that a user can only have one active\n session at a time.\nSynonyms:\npevensie/auth.create_session\npevensie/auth create_session","url":"pevensie/auth.html#create_session"},{"doc":"pevensie/auth","title":"create_session_cookie","content":"pub fn create_session_cookie(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  session session: Session,\n) -&gt; Result(String, Nil)\n Create a signed cookie for a session.\nSynonyms:\npevensie/auth.create_session_cookie\npevensie/auth create_session_cookie","url":"pevensie/auth.html#create_session_cookie"},{"doc":"pevensie/auth","title":"create_user_with_email","content":"pub fn create_user_with_email(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  email email: String,\n  password password: String,\n  user_metadata user_metadata: c,\n) -&gt; Result(User(c), CreateError(b))\n Create a new user with the given email and password.\nSynonyms:\npevensie/auth.create_user_with_email\npevensie/auth create_user_with_email","url":"pevensie/auth.html#create_user_with_email"},{"doc":"pevensie/auth","title":"default_user_search_fields","content":"pub fn default_user_search_fields() -&gt; UserSearchFields\n A convenience function to create a [`UserSearchFields`](#UserSearchFields)\n with all fields set to `None`.\n\n \n import pevensie/user\n\n fn main() {\n   // ...\n   user.list_users(\n     pevensie_auth,\n     user.UserSearchFields(\n       ..user.default_user_search_fields(),\n       email: Some([&quot;isaac@pevensie.dev&quot;]),\n     ),\n   )\n }\n \nSynonyms:\npevensie/auth.default_user_search_fields\npevensie/auth default_user_search_fields","url":"pevensie/auth.html#default_user_search_fields"},{"doc":"pevensie/auth","title":"default_user_update","content":"pub fn default_user_update() -&gt; UserUpdate(a)\n A convenience function to create a [`UserUpdate`](#UserUpdate)\n with all fields set to `Ignore`.\n\n \n import pevensie/auth\n import pevensie/user.{Set}\n\n fn main() {\n   // ...\n   auth.update_user(\n     pevensie_auth,\n     user.id,\n     user.UserUpdate(\n       ..user.default_user_update(),\n       email: Set(&quot;new_email@example.com&quot;),\n     ),\n   )\n }\n \nSynonyms:\npevensie/auth.default_user_update\npevensie/auth default_user_update","url":"pevensie/auth.html#default_user_update"},{"doc":"pevensie/auth","title":"delete_one_time_token","content":"pub fn delete_one_time_token(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n  token_type token_type: OneTimeTokenType,\n  token token: String,\n) -&gt; Result(Nil, DeleteError(b))\n\nSynonyms:\npevensie/auth.delete_one_time_token\npevensie/auth delete_one_time_token","url":"pevensie/auth.html#delete_one_time_token"},{"doc":"pevensie/auth","title":"delete_session","content":"pub fn delete_session(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  session_id session_id: String,\n) -&gt; Result(Nil, DeleteError(b))\n Delete a session by ID.\nSynonyms:\npevensie/auth.delete_session\npevensie/auth delete_session","url":"pevensie/auth.html#delete_session"},{"doc":"pevensie/auth","title":"disconnect","content":"pub fn disconnect(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n) -&gt; Result(\n  PevensieAuth(a, b, c, Disconnected),\n  DisconnectError(b),\n)\n Runs teardown for your chosen auth driver and returns a disconnected\n [`PevensieAuth`](#PevensieAuth) instance.\nSynonyms:\npevensie/auth.disconnect\npevensie/auth disconnect","url":"pevensie/auth.html#disconnect"},{"doc":"pevensie/auth","title":"get_session","content":"pub fn get_session(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  session_id session_id: String,\n  ip ip: Option(IpAddress),\n  user_agent user_agent: Option(String),\n) -&gt; Result(Session, GetError(b))\n Fetch a session by ID. If IP address or user agent were provided when\n the session was created, they should be provided here as well.\nSynonyms:\npevensie/auth.get_session\npevensie/auth get_session","url":"pevensie/auth.html#get_session"},{"doc":"pevensie/auth","title":"get_user_by_email","content":"pub fn get_user_by_email(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  email email: String,\n) -&gt; Result(User(c), GetError(b))\n Fetch a single user by email.\n\n Errors if exactly one user is not found.\nSynonyms:\npevensie/auth.get_user_by_email\npevensie/auth get_user_by_email","url":"pevensie/auth.html#get_user_by_email"},{"doc":"pevensie/auth","title":"get_user_by_email_and_password","content":"pub fn get_user_by_email_and_password(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  email email: String,\n  password password: String,\n) -&gt; Result(User(c), GetError(b))\n Fetch a single user by email and password.\n\n Errors if exactly one user is not found, or if the password for the\n user is incorrect.\nSynonyms:\npevensie/auth.get_user_by_email_and_password\npevensie/auth get_user_by_email_and_password","url":"pevensie/auth.html#get_user_by_email_and_password"},{"doc":"pevensie/auth","title":"get_user_by_id","content":"pub fn get_user_by_id(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n) -&gt; Result(User(c), GetError(b))\n Fetch a single user by ID.\n\n Errors if exactly one user is not found.\nSynonyms:\npevensie/auth.get_user_by_id\npevensie/auth get_user_by_id","url":"pevensie/auth.html#get_user_by_id"},{"doc":"pevensie/auth","title":"list_users","content":"pub fn list_users(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  limit limit: Int,\n  offset offset: Int,\n  filters filters: UserSearchFields,\n) -&gt; Result(List(User(c)), GetError(b))\n Retrieves a list of users based on the given search fields.\n\n The `filters` argument is a [`UserSearchFields`](/pevensie/user.html#UserSearchFields)\n type that contains the search fields to use. The `UserSearchFields` type\n contains a number of fields, such as `id`, `email`, and `phone_number`.\n Each field can be set to a list of values to search for, or to `None` to\n search for all values.\n\n Drivers may handle search fields differently, so see the documentation\n for your chosen driver for more information.\nSynonyms:\npevensie/auth.list_users\npevensie/auth list_users","url":"pevensie/auth.html#list_users"},{"doc":"pevensie/auth","title":"log_in_user","content":"pub fn log_in_user(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  email email: String,\n  password password: String,\n  ip ip: Option(IpAddress),\n  user_agent user_agent: Option(String),\n) -&gt; Result(#(Session, User(c)), LogInError(b))\n Log in a user using email and password.\n\n Assuming credentials are valid, this function will create a new session\n for the user, and update the user&#39;s last sign in time to the current UTC time.\n\n Uses a session TTL of 24 hours, and does not delete any other sessions\n for the user.\nSynonyms:\npevensie/auth.log_in_user\npevensie/auth log_in_user","url":"pevensie/auth.html#log_in_user"},{"doc":"pevensie/auth","title":"new","content":"pub fn new(\n  driver driver: AuthDriver(a, b, c),\n  user_metadata_decoder user_metadata_decoder: fn(Dynamic) -&gt;\n    Result(c, List(DecodeError)),\n  user_metadata_encoder user_metadata_encoder: fn(c) -&gt; Json,\n  cookie_key cookie_key: String,\n) -&gt; PevensieAuth(a, b, c, Disconnected)\n Creates a new [`PevensieAuth`](#PevensieAuth) instance.\n\n The `driver` argument is the driver to use for authentication. This\n should be the driver that you&#39;ve created using the `new_auth_driver`\n function.\n\n The `user_metadata_decoder` and `user_metadata_encoder` arguments are\n used to decode and encode user metadata. These should be the inverse\n of each other, and should be able to handle both decoding and encoding\n user metadata to JSON.\n\n The `cookie_key` argument is used to sign and verify cookies. It should\n be a long, random string. It&#39;s recommended to use a secret key from\n a cryptographically secure source, and store it in a secure location.\nSynonyms:\npevensie/auth.new\npevensie/auth new","url":"pevensie/auth.html#new"},{"doc":"pevensie/auth","title":"set_user_email","content":"pub fn set_user_email(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n  email email: String,\n) -&gt; Result(User(c), UpdateError(b))\n Update a user&#39;s email.\nSynonyms:\npevensie/auth.set_user_email\npevensie/auth set_user_email","url":"pevensie/auth.html#set_user_email"},{"doc":"pevensie/auth","title":"set_user_password","content":"pub fn set_user_password(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n  password password: Option(String),\n) -&gt; Result(User(c), UpdateError(b))\n Update a user&#39;s password.\n\n If a password is provided, it will be hashed using Argon2 before being\n stored in the database. If no password is provided, the user&#39;s password\n will be set to `null`.\nSynonyms:\npevensie/auth.set_user_password\npevensie/auth set_user_password","url":"pevensie/auth.html#set_user_password"},{"doc":"pevensie/auth","title":"set_user_role","content":"pub fn set_user_role(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n  role role: Option(String),\n) -&gt; Result(User(c), UpdateError(b))\n Update a user&#39;s role.\nSynonyms:\npevensie/auth.set_user_role\npevensie/auth set_user_role","url":"pevensie/auth.html#set_user_role"},{"doc":"pevensie/auth","title":"update_last_sign_in","content":"pub fn update_last_sign_in(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n  last_sign_in last_sign_in: Option(Time),\n) -&gt; Result(User(c), UpdateError(b))\n Update a user&#39;s last sign in time.\nSynonyms:\npevensie/auth.update_last_sign_in\npevensie/auth update_last_sign_in","url":"pevensie/auth.html#update_last_sign_in"},{"doc":"pevensie/auth","title":"update_user","content":"pub fn update_user(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n  user_update user_update: UserUpdate(c),\n) -&gt; Result(User(c), UpdateError(b))\n Update a user by ID. See [`UserUpdate`](/pevensie/user.html#UserUpdate)\n for more information on how to provide the fields to be updated.\n\n \n import pevensie/auth.{type PevensieAuth}\n\n pub fn main() {\n   // ...\n   let assert Ok(user) = auth.update_user(\n     pevensie_auth,\n     user.id,\n     UserUpdate(..user.default_user_update(), email: Set(&quot;new_email@example.com&quot;)),\n   )\n   // ...\n }\n \n\n &gt; Note: if updating the user&#39;s password, you should use the\n &gt; [`set_user_password`](#set_user_password) function instead in order\n &gt; to hash the password before storing it in the database.\nSynonyms:\npevensie/auth.update_user\npevensie/auth update_user","url":"pevensie/auth.html#update_user"},{"doc":"pevensie/auth","title":"update_user_metadata","content":"pub fn update_user_metadata(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n  user_metadata user_metadata: c,\n) -&gt; Result(User(c), UpdateError(b))\n Update user metadata.\nSynonyms:\npevensie/auth.update_user_metadata\npevensie/auth update_user_metadata","url":"pevensie/auth.html#update_user_metadata"},{"doc":"pevensie/auth","title":"use_one_time_token","content":"pub fn use_one_time_token(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n  token_type token_type: OneTimeTokenType,\n  token token: String,\n) -&gt; Result(Nil, UpdateError(b))\n\nSynonyms:\npevensie/auth.use_one_time_token\npevensie/auth use_one_time_token","url":"pevensie/auth.html#use_one_time_token"},{"doc":"pevensie/auth","title":"user_encoder","content":"pub fn user_encoder(\n  user: User(a),\n  user_metadata_encoder: fn(a) -&gt; Json,\n) -&gt; Json\n Encodes a [`User`](#User) value to JSON.\nSynonyms:\npevensie/auth.user_encoder\npevensie/auth user_encoder","url":"pevensie/auth.html#user_encoder"},{"doc":"pevensie/auth","title":"validate_one_time_token","content":"pub fn validate_one_time_token(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  user_id user_id: String,\n  token_type token_type: OneTimeTokenType,\n  token token: String,\n) -&gt; Result(Nil, GetError(b))\n\nSynonyms:\npevensie/auth.validate_one_time_token\npevensie/auth validate_one_time_token","url":"pevensie/auth.html#validate_one_time_token"},{"doc":"pevensie/auth","title":"validate_session_cookie","content":"pub fn validate_session_cookie(\n  pevensie_auth: PevensieAuth(a, b, c, Connected),\n  cookie cookie: String,\n) -&gt; Result(String, Nil)\n Validate a signed cookie for a session. Returns the session ID if the\n cookie is valid.\nSynonyms:\npevensie/auth.validate_session_cookie\npevensie/auth validate_session_cookie","url":"pevensie/auth.html#validate_session_cookie"},{"doc":"pevensie/auth","title":"pevensie/auth","content":" Pevensie makes it simple to add authentication to your Gleam applications.\n Currently only email/password authentication is supported, but more\n authentication methods (OAuth2, passkeys, etc.) are planned for the future.\n\n While you can use Pevensie Auth without the rest of the Pevensie ecosystem,\n it&#39;s better used as a foundation on which to build a Pevensie-driven\n application. Other Pevensie modules are designed to integrate well with\n Pevensie Auth, so you can use them without worrying about authentication.\n\n ## Getting Started\n\n Pevensie Auth is driver-based, like many other Pevensie modules. This means\n that you need to choose a driver for your authentication needs. Pevensie\n provides in-house drivers, but hopefully in the future other drivers will\n be available.\n\n To get started, you&#39;ll need to create a type to represent your user\n metadata (see [`pevensie/user`](/pevensie/user) for more details), as well\n as a decoder and encoder for that type.\n\n ```gleam\n import gleam/dynamic.{type DecodeError}\n import gleam/json\n\n pub type UserMetadata {\n   UserMetadata(name: String, age: Int)\n }\n\n pub fn user_metadata_decoder() -&gt; Result(UserMetadata, List(DecodeError)) {\n   // ...\n }\n\n pub fn user_metadata_encoder(user_metadata: UserMetadata) -&gt; json.Json {\n   // ...\n }\n ```\n\n Next, you&#39;ll need to create a driver of your choice. Here, we&#39;ll be using the\n first-party Postgres driver, but you can use any driver you like.\n\n ```gleam\n import pevensie/drivers/postgres.{type PostgresConfig}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     database: &quot;my_database&quot;,\n   )\n   let driver = postgres.new_auth_driver(config)\n   // ...\n }\n ```\n\n Now that you have a driver, you can create a Pevensie Auth instance. This\n instance will act as your entrypoint to the Pevensie Auth API.\n\n ```gleam\n import pevensie/auth.{type PevensieAuth}\n\n pub fn main() {\n   // ...\n   let driver = postgres.new_auth_driver(config)\n   let pevensie_auth = auth.new(\n     driver:,\n     user_metadata_decoder:,\n     user_metadata_encoder:,\n     cookie_key: &quot;super secret signing key&quot;,\n   )\n   // ...\n }\n ```\n\n Make sure to call [`connect`](/pevensie/auth.html#connect) on your\n Pevensie Auth instance before using it. This allows your driver to\n perform any setup required, such as creating a database connection pool.\n Different drivers may require different setup and at different times\n (at the start of the application, once per request, etc.). See the\n documentation for your chosen driver for more information.\n\n Finally, create your first user using\n [`create_user_with_email`](/pevensie/auth.html#create_user_with_email).\n\n ```gleam\n import pevensie/auth.{type PevensieAuth}\n\n pub fn main() {\n   // ...\n   let assert Ok(pevensie_auth) = auth.connect(pevensie_auth)\n   auth.create_user_with_email(\n     pevensie_auth,\n     &quot;lucy@pevensie.dev&quot;,\n     &quot;password&quot;,\n     UserMetadata(name: &quot;Lucy Pevensie&quot;, age: 8),\n   )\n   // ...\n }\n ```\n\n ### Logging In Users\n\n Pevensie Auth provides individual functions for verifying user credentials\n and creating sessions. However, it also provides a convenience function\n for logging in users, which will create a session and update the user&#39;s\n last sign in time.\n\n ```gleam\n import pevensie/auth.{type PevensieAuth}\n\n pub fn main() {\n   // ...\n   let assert Ok(#(session, user)) = auth.log_in_user(\n     pevensie_auth,\n     &quot;lucy@pevensie.dev&quot;,\n     &quot;password&quot;,\n     Some(net.parse_ip_address(&quot;127.1&quot;)),\n     None,\n   )\n   // ...\n }\n ```\n\n ## Drivers\n\n Pevensie Auth is designed to be driver-agnostic, so you can use any driver\n you like. The drivers provided with Pevensie are:\n\n - [`postgres`](/pevensie/drivers/postgres.html) - A driver for PostgreSQL\n\n The hope is that other first- and third-party drivers will be available\n in the future.\n\n &gt; Note: database-based drivers will require migrations to be run before\n &gt; using them. See the documentation for your chosen driver for more\n &gt; information.\n\n ## Users\n\n A &#39;user&#39; in Pevensie is someone with a user ID. Users can be identified by ID,\n and optionally by email or phone number.\n\n The `User` type contains some metadata about a user, such as their role, email,\n and password hash. This metadata is used by Pevensie to store information about\n users, such as their last sign in time, and their role in the system.\n\n &gt; Note: this module only contains the `User` type and adjecent types and functions.\n &gt; If you wish to interact with users, you should use the\n &gt; [`pevensie/auth`](/pevensie/auth.html) module.\n\n ### Custom User Metadata\n\n Custom user metadata can be added to the `User` type using the `user_metadata`\n field. This field is best used for storing data specific to your application,\n such as user preferences or usernames. This is done to reduce the chance you&#39;ll\n need a separate database table/schema for storing your user data.\n\n Different drivers may handle user metadata differently, though the general\n recommendation is to ensure your user metadata can be encoded to JSON (even\n if it&#39;s not a direct mapping).\n\n For example, if you have a user metadata type that contains a date:\n\n ```gleam\n pub type UserMetadata {\n   UserMetadata(birthday: birl.Time)\n }\n ```\n\n You can encode this to JSON using the `user_metadata_encoder` function:\n\n ```gleam\n pub fn user_metadata_encoder(user_metadata: UserMetadata) -&gt; json.Json {\n   json.object([\n     #(&quot;birthday&quot;, json.string(birl.to_iso8601(user_metadata.birthday))),\n   ])\n }\n ```\n\n Your decoder can then handle decoding the JSON to a `Time` type, despite the\n fact that the JSON doesn&#39;t support dates.\n\n ## Session Management\n\n Pevensie Auth provides a simple API for managing sessions. Sessions are\n managed using opaque session IDs, which are tied to a user ID and optionally\n an IP address and user agent.\n\n Sessions are created using the\n [`create_session`](/pevensie/auth.html#create_session) function, and\n retrieved using the [`get_session`](/pevensie/auth.html#get_session)\n function. Sessions can be deleted using the\n [`delete_session`](/pevensie/auth.html#delete_session) function. Expired\n sessions may be deleted automatically by the driver, or they may be\n deleted manually using the\n [`delete_session`](/pevensie/auth.html#delete_session) function.\n\n ```gleam\n import gleam/option.{None, Some}\n import pevensie/auth.{type PevensieAuth}\n import pevensie/net\n\n pub fn main() {\n   // ...\n   let session = auth.create_session(\n     pevensie_auth,\n     user.id,\n     Some(net.parse_ip_address(&quot;127.1&quot;)),\n     None,\n     Some(24 * 60 * 60),\n   )\n   // ...\n }\n ```\n\n ### Cookies\n\n Session tokens should be provided as cookies, and Pevensie Auth provides\n convenience functions for signing and verifying cookies. You can choose not\n to sign cookies, but it&#39;s generally recommended to do so.","url":"pevensie/auth.html"},{"doc":"pevensie/cache","title":"CacheDriver","content":"pub type CacheDriver(driver, driver_error) {\n  CacheDriver(\n    driver: driver,\n    connect: ConnectFunction(driver, driver_error),\n    disconnect: DisconnectFunction(driver, driver_error),\n    set: CacheSetFunction(driver, driver_error),\n    get: CacheGetFunction(driver, driver_error),\n    delete: CacheDeleteFunction(driver, driver_error),\n  )\n}\n\nCacheDriver(\n  driver: driver,\n  connect: ConnectFunction(driver, driver_error),\n  disconnect: DisconnectFunction(driver, driver_error),\n  set: CacheSetFunction(driver, driver_error),\n  get: CacheGetFunction(driver, driver_error),\n  delete: CacheDeleteFunction(driver, driver_error),\n)\n\n\nSynonyms:\npevensie/cache.CacheDriver\npevensie/cache CacheDriver","url":"pevensie/cache.html#CacheDriver"},{"doc":"pevensie/cache","title":"DeleteError","content":"pub type DeleteError(cache_driver_error) {\n  DeleteDriverError(cache_driver_error)\n}\n\nDeleteDriverError(cache_driver_error)\n\n\nSynonyms:\npevensie/cache.DeleteError\npevensie/cache DeleteError","url":"pevensie/cache.html#DeleteError"},{"doc":"pevensie/cache","title":"GetError","content":"pub type GetError(cache_driver_error) {\n  GetDriverError(cache_driver_error)\n  GotTooFewRecords\n  GotTooManyRecords\n}\n\nGetDriverError(cache_driver_error)\n\n\nGotTooFewRecords\n\n\nGotTooManyRecords\n\n\nSynonyms:\npevensie/cache.GetError\npevensie/cache GetError","url":"pevensie/cache.html#GetError"},{"doc":"pevensie/cache","title":"PevensieCache","content":"pub type PevensieCache(driver, driver_error, connected) {\n  PevensieCache(driver: CacheDriver(driver, driver_error))\n}\n The entrypoint to the Pevensie Cache API. This type is used when using\n the majority of the functions in `pevensie/cache`.\n\n You must connect your Pevensie Cache instance before using it. This\n allows your driver to perform any setup required, such as creating\n a database connection pool.\n\n Create a new `PevensieCache` instance using the\n [`new`](#new) function.\n\n \n import pevensie/cache.{type PevensieCache}\n\n pub fn main() {\n   let pevensie_cache = cache.new(\n     postgres.new_cache_driver(postgres.default_config()),\n   )\n   // ...\n }\n \nPevensieCache(driver: CacheDriver(driver, driver_error))\n\n\nSynonyms:\npevensie/cache.PevensieCache\npevensie/cache PevensieCache","url":"pevensie/cache.html#PevensieCache"},{"doc":"pevensie/cache","title":"SetError","content":"pub type SetError(cache_driver_error) {\n  SetDriverError(cache_driver_error)\n}\n\nSetDriverError(cache_driver_error)\n\n\nSynonyms:\npevensie/cache.SetError\npevensie/cache SetError","url":"pevensie/cache.html#SetError"},{"doc":"pevensie/cache","title":"connect","content":"pub fn connect(\n  pevensie_cache: PevensieCache(a, b, Disconnected),\n) -&gt; Result(PevensieCache(a, b, Connected), ConnectError(b))\n Runs setup for your chosen cache driver and returns a connected\n [`PevensieCache`](#PevensieCache) instance.\n\n This function must be called before using any other functions in\n the Pevensie Cache API. Attempting to use the API before calling\n `connect` will result in a compile error.\n\n \n import pevensie/cache.{type PevensieCache}\n\n pub fn main() {\n   // ...\n   let assert Ok(pevensie_cache) = cache.connect(pevensie_cache)\n   // ...\n }\n \nSynonyms:\npevensie/cache.connect\npevensie/cache connect","url":"pevensie/cache.html#connect"},{"doc":"pevensie/cache","title":"delete","content":"pub fn delete(\n  pevensie_cache: PevensieCache(a, b, Connected),\n  resource_type resource_type: String,\n  key key: String,\n) -&gt; Result(Nil, DeleteError(b))\n Delete a value from the cache.\nSynonyms:\npevensie/cache.delete\npevensie/cache delete","url":"pevensie/cache.html#delete"},{"doc":"pevensie/cache","title":"disconnect","content":"pub fn disconnect(\n  pevensie_cache: PevensieCache(a, b, Connected),\n) -&gt; Result(PevensieCache(a, b, Disconnected), DisconnectError(b))\n Runs teardown for your chosen cache driver and returns a disconnected\n [`PevensieCache`](#PevensieCache) instance.\n\n After calling this function, you can no longer use the cache driver\n unless you call [`connect`](#connect) again.\nSynonyms:\npevensie/cache.disconnect\npevensie/cache disconnect","url":"pevensie/cache.html#disconnect"},{"doc":"pevensie/cache","title":"get","content":"pub fn get(\n  pevensie_cache: PevensieCache(a, b, Connected),\n  resource_type resource_type: String,\n  key key: String,\n) -&gt; Result(String, GetError(b))\n Retrieve a value from the cache.\nSynonyms:\npevensie/cache.get\npevensie/cache get","url":"pevensie/cache.html#get"},{"doc":"pevensie/cache","title":"new","content":"pub fn new(\n  driver driver: CacheDriver(a, b),\n) -&gt; PevensieCache(a, b, Disconnected)\n Creates a new [`PevensieCache`](#PevensieCache) instance.\n\n The `driver` argument is the driver to use for caching. Use either\n a driver provided by Pevensie, or any third-party driver you like.\n\n \n import pevensie/cache.{type PevensieCache}\n\n pub fn main() {\n   let pevensie_cache = cache.new(\n     postgres.new_cache_driver(postgres.default_config()),\n   )\n   // ...\n }\n \nSynonyms:\npevensie/cache.new\npevensie/cache new","url":"pevensie/cache.html#new"},{"doc":"pevensie/cache","title":"set","content":"pub fn set(\n  pevensie_cache: PevensieCache(a, b, Connected),\n  resource_type resource_type: String,\n  key key: String,\n  value value: String,\n  ttl_seconds ttl_seconds: Option(Int),\n) -&gt; Result(Nil, SetError(b))\n Store a value in the cache. Both the key and value must be strings.\n\n The `resource_type` argument is used to help organize your cache. It\n should be a string that uniquely identifies the type of data you&#39;re\n caching. For example, if you&#39;re caching user data, you might use\n `&quot;user&quot;` as the resource type.\n\n The `ttl_seconds` argument is the number of seconds the value should\n be stored in the cache. If `None`, the value will never expire.\n\n \n import pevensie/cache.{type PevensieCache}\n\n pub fn main() {\n   // ...\n   let assert Ok(pevensie_cache) = cache.connect(pevensie_cache)\n   let assert Ok(_) = cache.set(\n     pevensie_cache,\n     &quot;ship&quot;,\n     &quot;dawntreader&quot;,\n     &quot;{\\&quot;name\\&quot;: \\&quot;Dawn Treader\\&quot;, \\&quot;captain\\&quot;: \\&quot;Prince Caspian\\&quot;}&quot;,\n     None,\n   )\n   // ...\n }\n \nSynonyms:\npevensie/cache.set\npevensie/cache set","url":"pevensie/cache.html#set"},{"doc":"pevensie/cache","title":"pevensie/cache","content":" Pevensie Cache is a smiple module for caching data in your application.\n It is designed to be easily used with all parts of Pevensie, and follows\n the driver-based architecture.\n\n &gt; Important: data stored in Pevensie Cache may be lost (e.g. on restart),\n &gt; and is not guaranteed to be durable. Do not use Pevensie Cache for data\n &gt; that you need to keep indefinitely.\n\n ## Getting Started\n\n All you need to provide to the `new` function is your chosen driver.\n\n ```gleam\n import pevensie/cache.{type PevensieCache}\n import pevensie/drivers/postgres.{type PostgresConfig}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     database: &quot;my_database&quot;,\n   )\n   let driver = postgres.new_cache_driver(config)\n   let pevensie_cache = cache.new(driver)\n }\n ```\n\n You&#39;ll need to run the [`connect`](/pevensie/cache.html#connect) function\n on your Pevensie Cache instance before using it. This allows your driver\n to perform any setup required, such as creating a database connection\n pool.\n\n Once connected, you can use the [`set`](/pevensie/cache.html#set),\n [`get`](/pevensie/cache.html#get), and [`delete`](/pevensie/cache.html#delete)\n functions to store, retrieve, and delete data from the cache.\n\n ```gleam\n import pevensie/cache.{type PevensieCache}\n\n pub fn main() {\n   // ...\nlet assert Ok(pevensie_cache) = cache.connect(pevensie_cache)\n   cache.set(pevensie_cache, &quot;key&quot;, &quot;value&quot;, None)\n   cache.get(pevensie_cache, &quot;key&quot;)\n   cache.delete(pevensie_cache, &quot;key&quot;)\n   // ...\n }\n ```\n\n ## Drivers\n\n Pevensie Cache is designed to be driver-agnostic, so you can use any driver\n you like. The drivers provided with Pevensie are:\n\n - [`postgres`](/pevensie/drivers/postgres.html) - A driver for PostgreSQL\n - [`redis`](/pevensie/drivers/redis.html) - A driver for Redis (coming soon)\n - [`ets`](/pevensie/drivers/ets.html) - A driver for ETS, Erlang&#39;s in-memory key-value store (coming soon)\n\n The hope is that other first- and third-party drivers will be available\n in the future.\n\n &gt; Note: database-based drivers will require migrations to be run before\n &gt; using them. See the documentation for your chosen driver for more\n &gt; information.","url":"pevensie/cache.html"},{"doc":"pevensie/drivers/postgres","title":"IpVersion","content":"pub type IpVersion {\n  Ipv4\n  Ipv6\n}\n An IP version for a [`PostgresConfig`](#PostgresConfig).\nIpv4\n\n\nIpv6\n\n\nSynonyms:\npevensie/drivers/postgres.IpVersion\npevensie/drivers/postgres IpVersion","url":"pevensie/drivers/postgres.html#IpVersion"},{"doc":"pevensie/drivers/postgres","title":"Postgres","content":"pub opaque type Postgres\n The Postgres driver.\n\nSynonyms:\npevensie/drivers/postgres.Postgres\npevensie/drivers/postgres Postgres","url":"pevensie/drivers/postgres.html#Postgres"},{"doc":"pevensie/drivers/postgres","title":"PostgresConfig","content":"pub type PostgresConfig {\n  PostgresConfig(\n    host: String,\n    port: Int,\n    database: String,\n    user: String,\n    password: Option(String),\n    ssl: Bool,\n    connection_parameters: List(#(String, String)),\n    pool_size: Int,\n    queue_target: Int,\n    queue_interval: Int,\n    idle_interval: Int,\n    trace: Bool,\n    ip_version: IpVersion,\n  )\n}\n Configuration for connecting to a Postgres database.\n\n Use the [`default_config`](/pevensie/drivers/postgres.html#default_config)\n function to get a default configuration for connecting to a local\n Postgres database with sensible concurrency defaults.\n\n \n import pevensie/drivers/postgres.{type PostgresConfig}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     host: &quot;db.pevensie.dev&quot;,\n     database: &quot;my_database&quot;,\n   )\n   // ...\n }\n \nPostgresConfig(\n  host: String,\n  port: Int,\n  database: String,\n  user: String,\n  password: Option(String),\n  ssl: Bool,\n  connection_parameters: List(#(String, String)),\n  pool_size: Int,\n  queue_target: Int,\n  queue_interval: Int,\n  idle_interval: Int,\n  trace: Bool,\n  ip_version: IpVersion,\n)\n\n\nSynonyms:\npevensie/drivers/postgres.PostgresConfig\npevensie/drivers/postgres PostgresConfig","url":"pevensie/drivers/postgres.html#PostgresConfig"},{"doc":"pevensie/drivers/postgres","title":"PostgresError","content":"pub type PostgresError {\n  ConstraintViolated(\n    message: String,\n    constraint: String,\n    detail: String,\n  )\n  PostgresqlError(code: String, name: String, message: String)\n  ConnectionUnavailable\n}\n Errors that can occur when interacting with the Postgres driver.\n Will probably be removed or changed - haven&#39;t decided on the final API yet.\nConstraintViolated(\n  message: String,\n  constraint: String,\n  detail: String,\n)\n\n\nPostgresqlError(code: String, name: String, message: String)\n\n\nConnectionUnavailable\n\n\nSynonyms:\npevensie/drivers/postgres.PostgresError\npevensie/drivers/postgres PostgresError","url":"pevensie/drivers/postgres.html#PostgresError"},{"doc":"pevensie/drivers/postgres","title":"session_select_fields","content":"pub const session_select_fields: String\n The SQL used to select fields from the `session` table.\nSynonyms:\npevensie/drivers/postgres.session_select_fields\npevensie/drivers/postgres session_select_fields","url":"pevensie/drivers/postgres.html#session_select_fields"},{"doc":"pevensie/drivers/postgres","title":"user_select_fields","content":"pub const user_select_fields: String\n The SQL used to select fields from the `user` table.\nSynonyms:\npevensie/drivers/postgres.user_select_fields\npevensie/drivers/postgres user_select_fields","url":"pevensie/drivers/postgres.html#user_select_fields"},{"doc":"pevensie/drivers/postgres","title":"default_config","content":"pub fn default_config() -&gt; PostgresConfig\n Returns a default [`PostgresConfig`](#PostgresConfig) for connecting to a local\n Postgres database.\n\n Can also be used to provide sensible concurrency defaults for connecting\n to a remote database.\n\n \n import pevensie/drivers/postgres.{type PostgresConfig}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     host: &quot;db.pevensie.dev&quot;,\n     database: &quot;my_database&quot;,\n   )\n   // ...\n }\n \nSynonyms:\npevensie/drivers/postgres.default_config\npevensie/drivers/postgres default_config","url":"pevensie/drivers/postgres.html#default_config"},{"doc":"pevensie/drivers/postgres","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\npevensie/drivers/postgres.main\npevensie/drivers/postgres main","url":"pevensie/drivers/postgres.html#main"},{"doc":"pevensie/drivers/postgres","title":"new_auth_driver","content":"pub fn new_auth_driver(\n  config: PostgresConfig,\n) -&gt; AuthDriver(Postgres, PostgresError, a)\n Creates a new [`AuthDriver`](/pevensie/drivers/drivers.html#AuthDriver) for use with\n the [`pevensie/auth.new`](/pevensie/auth.html#new) function.\n\n \n import pevensie/drivers/postgres.{type PostgresConfig}\n import pevensie/auth.{type PevensieAuth}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     host: &quot;db.pevensie.dev&quot;,\n     database: &quot;my_database&quot;,\n   )\n   let driver = postgres.new_auth_driver(config)\n   let pevensie_auth = auth.new(\n     driver:,\n     user_metadata_decoder:,\n     user_metadata_encoder:,\n     cookie_key: &quot;super secret signing key&quot;,\n   )\n   // ...\n }\n \nSynonyms:\npevensie/drivers/postgres.new_auth_driver\npevensie/drivers/postgres new_auth_driver","url":"pevensie/drivers/postgres.html#new_auth_driver"},{"doc":"pevensie/drivers/postgres","title":"new_cache_driver","content":"pub fn new_cache_driver(\n  config: PostgresConfig,\n) -&gt; CacheDriver(Postgres, PostgresError)\n Creates a new [`CacheDriver`](/pevensie/drivers/drivers.html#CacheDriver) for use with\n the [`pevensie/cache.new`](/pevensie/cache.html#new) function.\n\n \n import pevensie/drivers/postgres.{type PostgresConfig}\n import pevensie/cache.{type PevensieCache}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     host: &quot;db.pevensie.dev&quot;,\n     database: &quot;my_database&quot;,\n   )\n   let driver = postgres.new_cache_driver(config)\n   let pevensie_cache = cache.new(driver)\n   // ...\n }\n \nSynonyms:\npevensie/drivers/postgres.new_cache_driver\npevensie/drivers/postgres new_cache_driver","url":"pevensie/drivers/postgres.html#new_cache_driver"},{"doc":"pevensie/drivers/postgres","title":"session_decoder","content":"pub fn session_decoder() -&gt; fn(Dynamic) -&gt;\n  Result(Session, List(DecodeError))\n A decoder for the `session` table. Requires use of the\n [`session_select_fields`](#session_select_fields) when querying.\nSynonyms:\npevensie/drivers/postgres.session_decoder\npevensie/drivers/postgres session_decoder","url":"pevensie/drivers/postgres.html#session_decoder"},{"doc":"pevensie/drivers/postgres","title":"user_decoder","content":"pub fn user_decoder(\n  user_metadata_decoder: fn(Dynamic) -&gt;\n    Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(User(a), List(DecodeError))\n A decoder for the `user` table. Requires use of the\n [`user_select_fields`](#user_select_fields) when querying.\nSynonyms:\npevensie/drivers/postgres.user_decoder\npevensie/drivers/postgres user_decoder","url":"pevensie/drivers/postgres.html#user_decoder"},{"doc":"pevensie/drivers/postgres","title":"pevensie/drivers/postgres","content":" The official PostgreSQL driver for Pevensie. It provides driver\n implementations for Pevensie modules to be used with Postgres\n databases.\n\n Currently provides drivers for:\n\n - [Pevensie Auth](/pevensie/auth.html)\n - [Pevensie Cache](/pevensie/cache.html)\n\n ## Getting Started\n\n Configure your driver to connect to your database using the\n [`PostgresConfig`](/pevensie/drivers/postgres.html#PostgresConfig)\n type. You can use the [`default_config`](/pevensie/drivers/postgres.html#default_config)\n function to get a default configuration for connecting to a local\n Postgres database with sensible concurrency defaults.\n\n ```gleam\n import pevensie/drivers/postgres.{type PostgresConfig}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     host: &quot;db.pevensie.dev&quot;,\n     database: &quot;my_database&quot;,\n   )\n   // ...\n }\n ```\n\n Create a new driver using one of the `new_&lt;driver&gt;_driver` functions\n provided by this module. You can then use the driver with Pevensie\n modules.\n\n ```gleam\n import pevensie/drivers/postgres.{type PostgresConfig}\n import pevensie/auth.{type PevensieAuth}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     host: &quot;db.pevensie.dev&quot;,\n     database: &quot;my_database&quot;,\n   )\n   let driver = postgres.new_auth_driver(config)\n   let pevensie_auth = auth.new(\n     driver:,\n     user_metadata_decoder:,\n     user_metadata_encoder:,\n     cookie_key: &quot;super secret signing key&quot;,\n   )\n   // ...\n }\n ```\n\n ## Connection Management\n\n When called with the Postgres driver, the [`connect`](/pevensie/auth.html#connect) function\n provided by Pevensie Auth will create a connection pool for the database. This can be called\n once on boot, and will be reused for the lifetime of the application.\n\n The [`disconnect`](/pevensie/auth.html#disconnect) function will close the connection pool.\n\n ## Tables\n\n This driver creates tables in the `pevensie` schema to store user\n data, sessions, and cache data.\n\n The current tables created by this driver are:\n\n | Table Name | Description |\n | ---------- | ----------- |\n | `cache` | Stores cache data. This table is unlogged, so data will be lost when the database stops. |\n | `one_time_token` | Stores one time tokens. Uses soft deletions. |\n | `session` | Stores session data. |\n | `user` | Stores user data. Uses soft deletions. |\n | `module_version` | Stores the current versions of the tables required by this driver. Versions are stored as dates. |\n\n ### Types\n\n Generally, the Postgres driver makes a best effort to map the types\n used by Pevensie to best-practice Postgres types. Generally, columns\n are non-nullable unless the Gleam type is `Option(a)`. The following\n types are mapped to Postgres types:\n\n | Gleam Type | Postgres Type |\n | ---------- | ------------- |\n | Any resource ID | `UUID` (generated as UUIDv7) |\n | `String` | `text` |\n | `birl.Time` | `timestamptz` |\n | Record types (e.g. `user_metadata`) | `jsonb` |\n | `pevensie/net.IpAddr` | `inet` |\n\n ## Migrations\n\n You can run migrations against your\n database using the provided CLI:\n\n ```sh\n gleam run -m pevensie/drivers/postgres migrate --addr=&lt;connection_string&gt; auth cache\n ```\n\n The required SQL statements will be printed to the console. You can\n apply the migrations directly using the `--apply` flag.\n\n ## Implementation Details\n\n This driver uses the [gleam_pgo](https://github.com//pgo) library for interacting\n with Postgres.\n\n All IDs are stored as UUIDs, and are generated using using a UUIDv7 implementation\n made available by [Fabio Lima](https://github.com/fabiolimace) under the MIT license.\n The implementation is available [here](https://gist.github.com/fabiolimace/515a0440e3e40efeb234e12644a6a346).\n\n ### Pevensie Auth\n\n The `user` table follows the structure of the [`User`](/pevensie/user.html#User)\n type. The `user_metadata` column is a JSONB column, and is used to store\n any custom user metadata.\n\n It also contains a `deleted_at` column, which is used to mark users as deleted,\n rather than deleting the row from the database.\n\n Alongside the primary key, the `user` table has unique indexes on the\n `email` and `phone_number` columns. These are partial, and only index\n where values are provided. They also include the `deleted_at` column,\n so users can sign up with the same email if a user with that email\n has been deleted.\n\n The `session` table follows the structure of the [`Session`](/pevensie/session.html#Session)\n type. The `user_id` column is a foreign key referencing the `user` table. If\n an expired session is read, the `get_session` function will return `None`, and\n delete the expired session from the database.\n\n #### Searching for users\n\n The `list_users` function provided by Pevensie Auth allows you to search for users\n by ID, email or phone number. With the Postgres driver, the lists in you\n [`UserSearchFields`](/pevensie/user.html#UserSearchFields) argument are processed\n using a `like any()` where clause. This means that you can search for users\n by providing a list of values, and the driver will search for users with\n any of those values.\n\n You can also use Postgres `like` wildcards in your search values. For example,\n if you search for users with an email ending in `@example.com`, you can use\n `%@example.com` as the search value.\n\n ### Pevensie Cache\n\n The `cache` table is an [`unlogged`](https://www.postgresql.org/docs/current/sql-createtable.html#SQL-CREATETABLE-UNLOGGED)\n table. This ensures that writes are fast, but data is lost when the database\n is stopped. Do not use Pevensie Cache for data you need to keep indefinitely.\n\n The table&#39;s primary key is a composite of the `resource_type` and `key`\n columns. This allows you to store multiple values for a given key, and\n retrieve them all at once.\n\n Writes to the cache will overwrite any existing value for the given\n resource type and key. This driver does not keep a version history.\n\n If an expired value is read, the `get` function will return `None`, and\n delete the expired value from the cache.\n\n ### Custom Queries\n\n If you wish to query the `user` or `session` tables directly, this driver\n provides helper utilities for doing so.\n\n The [`user_decoder`](/pevensie/drivers/postgres.html#user_decoder) and\n [`session_decoder`](/pevensie/drivers/postgres.html#session_decoder) functions\n can be used to decode selections from the `user` and `session` tables, respectively.\n\n The [`user_select_fields`](/pevensie/drivers/postgres.html#user_select_fields)\n and [`session_select_fields`](/pevensie/drivers/postgres.html#session_select_fields)\n variables contain the SQL used to select fields from the `user` and `session`\n tables for use with the `user_decoder` and `session_decoder` functions.","url":"pevensie/drivers/postgres.html"}]);